from pathlib import Path
from typing import Optional

import pandas as pd

pangolin_cols = [
    ('taxon', 'Sample', 'Sample name'),
    (
        'lineage',
        'Pangolin Lineage',
        'Pangolin global SARS-CoV-2 lineage assignment based on pangoLEARN model. '
        'For more info, see https://github.com/cov-lineages/pangolin/#pangolearn-description',
    ),
    (
        'probability',
        'Lineage Assignment Probability',
        'Pangolin lineage assignment probability from multinomial logistic regression. For more info, '
        'see https://github.com/cov-lineages/pangolin/#pangolearn-description'
    ),
    (
        'pangoLEARN_version',
        'pangoLEARN Lineages Version',
        'Release version of pangoLEARN SARS-CoV-2 '
        'lineages information used for assignment.'),
    (
        'status',
        'Pangolin QC Status',
        'QC status of Pangolin lineage assignment, i.e. QC pass or fail'
    ),
    (
        'note',
        'Pangolin QC Note',
        'Issues reported with Pangolin lineage assignment such as too many Ns in the input sequence'
        ' (Pangolin will not call a lineage for sequences with over 50% (0.5) N-content) or '
        'if the sequence is too short (e.g. "seq_len:0")'
    ),
]


def find_pangolin_lineage_csv(basedir: Path) -> Optional[Path]:
    for p in basedir.rglob('pangolin.lineage_report.csv'):
        return p


def read_pangolin_csv(path: Path) -> pd.DataFrame:
    df = pd.read_csv(path)
    df.sort_values('taxon', inplace=True)

    df.rename(columns={x: y for x, y, _ in pangolin_cols}, inplace=True)
    df.set_index('Sample', inplace=True)
    return df


def get_info(basedir: Path,
             pangolin_lineage_csv: Optional[Path] = None) -> Optional[pd.DataFrame]:
    if pangolin_lineage_csv:
        return read_pangolin_csv(pangolin_lineage_csv)
    else:
        path = find_pangolin_lineage_csv(basedir)
        if path:
            return read_pangolin_csv(path)



import logging
import re
from pathlib import Path
from typing import List, Mapping

import pandas as pd
from Bio import SeqIO

from ..util import find_file_for_each_sample

SAMPLE_NAME_CLEANUP = [
        re.compile(r'\.AF0\.\d+'),
        '.consensus',
        '.fasta',
        '.fa',
    ]

GLOB_PATTERNS = [
    '**/ivar/**/*.consensus.fa*',
    '**/*.consensus.fa*'
]


logger = logging.getLogger(__name__)


def read_fasta(sample: str, path: Path) -> List[str]:
    out = []
    for rec in SeqIO.parse(path, format='fasta'):
        out += [f'>{sample}', str(rec.seq)]
    if len(out) > 2:
        logger.info(f'More than 1 sequence in consensus sequence for sample "{sample}". '
                    f'Appending sequence index starting at 1.')
        for idx, i in enumerate(range(0, len(out), 2)):
            out[i] = f'>{sample}-{idx + 1}'
    return out


def fasta_dataframe(sample_fasta: Mapping[str, Path]) -> pd.DataFrame:
    df = pd.DataFrame()
    fasta_list = []
    for sample in sorted(sample_fasta.keys()):
        fasta_path = sample_fasta[sample]
        fasta_list += read_fasta(sample, fasta_path)
    df['fasta'] = fasta_list
    return df


def get_info(basedir: Path) -> pd.DataFrame:
    sample_fasta = find_file_for_each_sample(basedir,
                                             glob_patterns=GLOB_PATTERNS,
                                             sample_name_cleanup=SAMPLE_NAME_CLEANUP)
    return fasta_dataframe(sample_fasta)
